/*******************************************************************************
 * Copyright (c) 2017 Tecnalia.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 	
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * The code has been contributed over a stub generated with the JHipster
 * Application generator http://www.jhipster.tech/.
 * 
 * Contributors (in alphabetical order):
 * 
 * Gorka Benguria                       Tecnalia
 * Iñaki Etxaniz                        Tecnalia
 * Juncal Alonso                        Tecnalia
 * Leire Orue-Echevarria                Tecnalia
 * Maria Jose lopez                     Tecnalia
 * Marisa Escalante                     Tecnalia
 * 
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 ******************************************************************************/
package eu.decideh2020.acsmi.backend.services.server.web.rest;

import java.net.URI;
import java.net.URISyntaxException;
import java.time.ZonedDateTime;
import java.util.List;
import java.util.Optional;

import javax.validation.Valid;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.codahale.metrics.annotation.Timed;
import eu.decideh2020.acsmi.backend.services.server.domain.ServiceClass;
import eu.decideh2020.acsmi.backend.services.server.repository.ServiceClassRepository;
import eu.decideh2020.acsmi.backend.services.server.web.rest.util.HeaderUtil;

import io.github.jhipster.web.util.ResponseUtil;

/**
 * REST controller for managing ServiceClass.
 */
@RestController
@RequestMapping("/api")
public class ServiceClassResource {

    private final Logger log = LoggerFactory.getLogger(ServiceClassResource.class);

    private static final String ENTITY_NAME = "serviceClass";
        
    private final ServiceClassRepository serviceClassRepository;

    public ServiceClassResource(ServiceClassRepository serviceClassRepository) {
        this.serviceClassRepository = serviceClassRepository;
    }

    /**
     * POST  /service-classes : Create a new serviceClass.
     *
     * @param serviceClass the serviceClass to create
     * @return the ResponseEntity with status 201 (Created) and with body the new serviceClass, or with status 400 (Bad Request) if the serviceClass has already an ID
     * @throws URISyntaxException if the Location URI syntax is incorrect
     */
    @PostMapping("/service-classes")
    @Timed
    @Transactional(noRollbackFor = Exception.class)
    public ResponseEntity<ServiceClass> createServiceClass(@Valid @RequestBody ServiceClass serviceClass) throws URISyntaxException {
    	log.debug("REST request to save ServiceClass : {}", serviceClass);
    	try {
            if (serviceClass.getId() != null) {
                return ResponseEntity.badRequest().headers(HeaderUtil.createFailureAlert(
                		ENTITY_NAME, "idexists", "A new serviceClass cannot already have an ID")).body(null);
            }

            // Comprobar que el nombre de la clase de servicio no exista previamente
            List<ServiceClass> serviceClasses = serviceClassRepository.findAll();
            if (serviceClasses != null && !serviceClasses.isEmpty()) {
            	for (ServiceClass cont : serviceClasses) {
            		if (serviceClass != null && serviceClass.getServiceclassname() != null
            				&& serviceClass.getServiceclassname().trim().equalsIgnoreCase(cont.getServiceclassname().trim())) {
            			return ResponseEntity.badRequest().headers(HeaderUtil.createFailureAlert(
                        		ENTITY_NAME, "serviceClassExists", "The ServiceClass is already registered")).body(null);
            		}
            	}
            }
            
            // Insertar la clase de servicio
            log.debug("createServiceClass  ::  creating ServiceClass...");
            ServiceClass result = serviceClassRepository.saveAndFlush(serviceClass);
        	log.debug("createServiceClass  ::  creating OK");
        	
            return ResponseEntity.created(new URI("/api/service-classes/" + result.getId()))
                .headers(HeaderUtil.createEntityCreationAlert(ENTITY_NAME, result.getServiceclassname()))
                .body(result);
    	} catch (Exception e) {
    		log.error("createServiceClass  ::  Exception");
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(ENTITY_NAME, "serviceClassCreateError", "No puede crearse la clase de servicio"))
                    .body(null);
    	}
    }

    /**
     * PUT  /service-classes : Updates an existing serviceClass.
     *
     * @param serviceClass the serviceClass to update
     * @return the ResponseEntity with status 200 (OK) and with body the updated serviceClass,
     * or with status 400 (Bad Request) if the serviceClass is not valid,
     * or with status 500 (Internal Server Error) if the serviceClass couldnt be updated
     * @throws URISyntaxException if the Location URI syntax is incorrect
     */
    @PutMapping("/service-classes")
    @Timed
    @Transactional
    public ResponseEntity<ServiceClass> updateServiceClass(@Valid @RequestBody ServiceClass serviceClass) throws URISyntaxException {
    	log.debug("REST request to update ServiceClass : {}", serviceClass);
    	try {
    		boolean reactivate = false;
    		
    		// Si la fecha de baja no es nula, se trata de una reactivación
    		if (serviceClass.getDeleteddate() != null) {
    			log.debug("updateServiceClass  ::  reactivating ServiceClass...");
    			reactivate = true;
    			serviceClass.setDeleteddate(null);
    		} else {
    			if (serviceClass.getId() == null) {
                    return createServiceClass(serviceClass);
                }
        		
        		// Comprobar que la clase de servicio no exista previamente
                List<ServiceClass> serviceClasses = serviceClassRepository.findAll();
                if (serviceClasses != null && !serviceClasses.isEmpty()) {
                	for (ServiceClass cont : serviceClasses) {
                		if (serviceClass != null && serviceClass.getServiceclassname() != null
                				&& serviceClass.getServiceclassname().trim().equalsIgnoreCase(cont.getServiceclassname().trim())
                				&& serviceClass.getId().longValue() != cont.getId().longValue()) {
                			return ResponseEntity.badRequest().headers(HeaderUtil.createFailureAlert(
                            		ENTITY_NAME, "serviceClassExists", "The ServiceClass is already registered")).body(null);
                		}
                	}
                }
    		}
                
    		// Modificar el tipo de servicio
    		log.debug("updateServiceClass  ::  updating ServiceClass...");
    		ServiceClass result = serviceClassRepository.saveAndFlush(serviceClass);
    		log.debug("updateServiceClass  ::  updating OK");
    		return reactivate
    				? ResponseEntity.ok().headers(HeaderUtil.createEntityReactivateAlert(ENTITY_NAME, serviceClass.getServiceclassname())).body(result)
    						: ResponseEntity.ok().headers(HeaderUtil.createEntityUpdateAlert(ENTITY_NAME, serviceClass.getServiceclassname())).body(result);
        } catch (Exception e) {
        	log.error("updateServiceClass  ::  Exception");
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(ENTITY_NAME, "serviceClassUpdateError", "No puede actualizarse la Clase de servicio"))
                    .body(null);
        }
    }

    /**
     * GET  /service-classes : get all the serviceClasses.
     *
     * @return the ResponseEntity with status 200 (OK) and the list of serviceClasses in body
     */
    @GetMapping("/service-classes")
    @Timed
    @Transactional
    public List<ServiceClass> getAllServiceClasses() {
       log.debug("REST request to get all ServiceClasses");
        try {log.debug("getAllServiceClasses  ::  getting all Service Classes...");
        	return serviceClassRepository.findAll();
        } catch (Exception e) {
        	log.error("getAllServiceClasses  ::  Exception");
        	return null;
        }
    }

    /**
     * GET  /service-classes/:id : get the "id" serviceClass.
     *
     * @param id the id of the serviceClass to retrieve
     * @return the ResponseEntity with status 200 (OK) and with body the serviceClass, or with status 404 (Not Found)
     */
    @GetMapping("/service-classes/{id}")
    @Timed
    public ResponseEntity<ServiceClass> getServiceClass(@PathVariable Long id) {
        log.debug("REST request to get ServiceClass : {}", id);
        ServiceClass serviceClass = null;
        try {
        	log.debug("getServiceClass  ::  getting service class...");
        	serviceClass = serviceClassRepository.findOne(id);
        	
        	log.debug("getServiceClass  ::  getting OK");
            return ResponseUtil.wrapOrNotFound(Optional.ofNullable(serviceClass));
        } catch (Exception e) {
        	log.error("getServiceClass  ::  Exception");
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(ENTITY_NAME, "serviceClassGetError", "No puede obtenerse la clase de servicio"))
                    .body(null);
        }
    }

    /**
     * DELETE  /service-classes/:id : delete the "id" serviceClass.
     *
     * @param id the id of the serviceClass to delete
     * @return the ResponseEntity with status 200 (OK)
     */
    @DeleteMapping("/service-classes/{id}")
    @Timed
    @Transactional
    public ResponseEntity<Void> deleteServiceClass(@PathVariable Long id, @RequestParam String forcedelete) {
    	log.debug("REST request to delete ServiceClass : {}", id);
        try {
        	log.debug("deleteServiceClass  ::  forcedelete=" + forcedelete);
        	
        	// Obtener la clase de servicio
        	ServiceClass serviceClass = serviceClassRepository.findOne(id);
        	boolean markAsErased = false;
        	
        	if (forcedelete != null && !Boolean.valueOf(forcedelete).booleanValue()) {
        		// Marcar la clase de servicio como borrado
            	if (serviceClass != null) {
    	        	log.debug("deleteServiceClass  ::  soft deleting ServiceClass...");
    	        	markAsErased = true;
    	        	serviceClass.setDeleteddate(ZonedDateTime.now());
    	        	serviceClassRepository.saveAndFlush(serviceClass);
            	}
        	} else {
        		// Eliminar definitivamente la clase de servicio
        		log.debug("deleteServiceClass  ::  deleting service class...");
        		serviceClassRepository.delete(id);
        	}
        	
        	log.debug("deleteServiceClass  ::  deleting OK");
            return markAsErased
            		? ResponseEntity.ok().headers(HeaderUtil.createEntitySoftDeletionAlert(ENTITY_NAME, serviceClass.getServiceclassname())).build()
            				: ResponseEntity.ok().headers(HeaderUtil.createEntityDeletionAlert(ENTITY_NAME, serviceClass.getServiceclassname())).build();
        } catch (EmptyResultDataAccessException e) {
            return ResponseEntity.noContent().build();
        } catch (Exception e) {
        	log.error("deleteServiceClass  ::  Exception  ::  " + e.getMessage());
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(ENTITY_NAME, "serviceClassDeleteError", "La clase de servicio no puede ser eliminado"))
                    .body(null);
        }
    }
}
