/*******************************************************************************
 * Copyright (c) 2017 Tecnalia.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 	
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * The code has been contributed over a stub generated with the JHipster
 * Application generator http://www.jhipster.tech/.
 * 
 * Contributors (in alphabetical order):
 * 
 * Gorka Benguria                       Tecnalia
 * Iñaki Etxaniz                        Tecnalia
 * Juncal Alonso                        Tecnalia
 * Leire Orue-Echevarria                Tecnalia
 * Maria Jose lopez                     Tecnalia
 * Marisa Escalante                     Tecnalia
 * 
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 ******************************************************************************/
package eu.decideh2020.acsmi.backend.services.server.web.rest;

import java.net.URI;
import java.net.URISyntaxException;
import java.time.ZonedDateTime;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Optional;
import java.util.Set;

import javax.validation.Valid;

import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.codahale.metrics.annotation.Timed;
import eu.decideh2020.acsmi.backend.services.server.domain.Service;
import eu.decideh2020.acsmi.backend.services.server.domain.ServiceAttributeValue;
import eu.decideh2020.acsmi.backend.services.server.domain.vo.ServiceAttributeVO;
import eu.decideh2020.acsmi.backend.services.server.domain.vo.ServiceFilterVO;
import eu.decideh2020.acsmi.backend.services.server.repository.ServiceAttributeValueRepository;
import eu.decideh2020.acsmi.backend.services.server.repository.ServiceRepository;
import eu.decideh2020.acsmi.backend.services.server.web.rest.util.HeaderUtil;

import io.github.jhipster.web.util.ResponseUtil;

/**
 * REST controller for managing Service.
 */
@RestController
@RequestMapping("/api")
public class ServiceResource {

    private final Logger log = LoggerFactory.getLogger(ServiceResource.class);

    private static final String ENTITY_NAME = "service";
    private static final String PARAM_SEPARATOR = "%#%";
    private static final String ATTRIBUTE_SEPARATOR = ">:";
        
    private final ServiceRepository serviceRepository;
    private final ServiceAttributeValueRepository serviceAttributeValueRepository;

    public ServiceResource(ServiceRepository serviceRepository, ServiceAttributeValueRepository serviceAttributeValueRepository) {
        this.serviceRepository = serviceRepository;
        this.serviceAttributeValueRepository = serviceAttributeValueRepository;
    }

    /**
     * POST  /services : Create a new service.
     *
     * @param service the service to create
     * @return the ResponseEntity with status 201 (Created) and with body the new service, or with status 400 (Bad Request) if the service has already an ID
     * @throws URISyntaxException if the Location URI syntax is incorrect
     */
    @PostMapping("/services")
    @Timed
    @Transactional
    public ResponseEntity<Service> createService(@Valid @RequestBody Service service, @RequestParam List<String> attributes) throws URISyntaxException {
    	log.debug("REST request to save Service : {}", service);
        try {
        	if (service.getId() != null) {
                return ResponseEntity.badRequest().headers(HeaderUtil.createFailureAlert(
                		ENTITY_NAME, "idexists", "A new service cannot already have an ID")).body(null);
            }
        	
        	// Comprobar que el nombre del servicio no exista previamente
            List<Service> services = serviceRepository.findAll();
            if (services != null && !services.isEmpty()) {
            	for (Service cont : services) {
            		if (service != null && service.getServicename() != null
            				&& service.getServicename().trim().equalsIgnoreCase(cont.getServicename().trim())) {
            			return ResponseEntity.badRequest().headers(HeaderUtil.createFailureAlert(
                        		ENTITY_NAME, "serviceNameExists", "The Service name is already registered")).body(null);
            		}
            	}
            }
        	
            // Insertar el servicio
        	log.debug("createService  ::  creating Service...");
        	Service result = serviceRepository.saveAndFlush(service);
        	
        	// Insertar los valores de atributos
        	if (attributes != null && attributes.size() > 1) {
        		ServiceAttributeValueResource attrResource = new ServiceAttributeValueResource(serviceAttributeValueRepository);
        		final int len = attributes.size();
        		for (int i = 0;i < len;i += 2) {
	        		ServiceAttributeValue value = new ServiceAttributeValue();
	        		value.setServiceattributevalue(attributes.get(i + 1));
	        		value.setServiceid(result.getId());
	        		value.setServiceattributetypeid(new Long(attributes.get(i)));
	        		
	        		attrResource.createServiceAttributeValue(value);
        		}
        	}
        	log.debug("createService  ::  creating OK");
        	
            return ResponseEntity.created(new URI("/api/services/" + result.getId()))
                .headers(HeaderUtil.createEntityCreationAlert(ENTITY_NAME, result.getServicename()))
                .body(result);
        } catch (Exception e) {
        	log.error("createService  ::  Exception  ::  " + e.getMessage());
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(ENTITY_NAME, "serviceCreateError", "No puede crearse el Servicio"))
                    .body(null);
        }
    }

    /**
     * PUT  /services : Updates an existing service.
     *
     * @param service the service to update
     * @return the ResponseEntity with status 200 (OK) and with body the updated service,
     * or with status 400 (Bad Request) if the service is not valid,
     * or with status 500 (Internal Server Error) if the service couldnt be updated
     * @throws URISyntaxException if the Location URI syntax is incorrect
     */
    @PutMapping("/services")
    @Timed
    @Transactional
    public ResponseEntity<Service> updateService(@Valid @RequestBody Service service, @RequestParam List<String> attributes) throws URISyntaxException {
    	log.debug("REST request to update Service : {}", service);
        try {
        	boolean reactivate = false;
        	
        	// Si la fecha de baja no es nula, se trata de una reactivación
        	if (service.getDeleteddate() != null) {
        		log.debug("updateService  ::  reactivating Service...");
        		reactivate = true;
        		
        		// Reactivar atributos del servicio
	        	serviceAttributeValueRepository.updateDeletedDateServiceAttributeValueByIdService(null, service.getId());
	        	
        		service.setDeleteddate(null);
        	} else {
        		if (service.getId() == null) {
                    return createService(service, attributes);
                }
        		
        		// Eliminar atributos y posteriormente insertarlos
        		serviceAttributeValueRepository.deleteServiceAttributeValueByIdService(service.getId());
        		
        		// Insertar los valores de atributos
        		ServiceAttributeValueResource attrResource = new ServiceAttributeValueResource(serviceAttributeValueRepository);
        		if (attributes != null && attributes.size() > 1) {
            		final int len = attributes.size();
            		for (int i = 0;i < len;i += 2) {
    	        		ServiceAttributeValue value = new ServiceAttributeValue();
    	        		value.setServiceattributevalue(attributes.get(i + 1));
    	        		value.setServiceid(service.getId());
    	        		value.setServiceattributetypeid(new Long(attributes.get(i)));
    	        		
    	        		attrResource.createServiceAttributeValue(value);
            		}
            	}
        	}
        	
        	// Modificar el servicio
        	log.debug("updateService  ::  updating Service...");
        	Service result = serviceRepository.saveAndFlush(service);
        	log.debug("updateService  ::  updating OK");
        	
            return reactivate
            		? ResponseEntity.ok().headers(HeaderUtil.createEntityReactivateAlert(ENTITY_NAME, service.getServicename())).body(result)
            				: ResponseEntity.ok().headers(HeaderUtil.createEntityUpdateAlert(ENTITY_NAME, service.getServicename())).body(result);
        } catch (Exception e) {
        	log.error("updateService  ::  Exception  ::  " + e.getMessage());
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(ENTITY_NAME, "serviceUpdateError", "No puede actualizarse el Servicio"))
                    .body(null);
        }
    }

    /**
     * GET  /services : get all the services.
     *
     * @return the ResponseEntity with status 200 (OK) and the list of services in body
     */
    @GetMapping("/services")
    @Timed
    public List<Service> getAllServices(@RequestParam String filter) {
    	log.debug("REST request to get all Services");
        try {
        	List<Service> list = null;
        	if (filter == null || filter.trim().length() == 0) {
        		log.debug("getAllServices  ::  getting all Services...");
        		list = serviceRepository.findAll();
        	} else {
        		// Construir objeto de filtro        			
       			ServiceFilterVO in = buildFilter(filter);
       			if (in != null) {
       				log.debug("getAllServices  ::  getting all Services filtering by  ::  " + filter);
       				
       				// Ejecutar query de filtro
       				List<ServiceFilterVO> listFilter = filterQuery(in);
       				if (listFilter != null && !listFilter.isEmpty()) {
       					// Construir lista de ids de servicios encontrados
            			List<Long> listIdservices = new ArrayList<Long>();
            			List<Long> listAttributeTypes = new ArrayList<Long>();
            			List<ServiceAttributeVO> listServicesAttributes = new ArrayList<ServiceAttributeVO>();
            			
            			// Recorrer resultado del filtrado
                		for (ServiceFilterVO out : listFilter) {
                			out.setAttributeTypeids(in.getAttributetypeids());
                			out.setAttributeTypevalues(in.getAttributetypevalues());

                			// Filtrar los valores introducidos
                			if (out.getAttributetypevalues().isEmpty()) {
                				log.debug("getAllServices  ::  Filtrado solo por clase de servicio");
                				listIdservices.add(out.getServiceid());
                			} else {
                				final int valuesLength = out.getAttributetypevalues().size();
                				for (int i = 0;i < valuesLength;i++) {
                					if (out.getTypeid().longValue() == out.getAttributetypeids().get(i).longValue()) {
                						// Añadir tipo de atributo a la lista, sea de una enumeración o no
                						listAttributeTypes.add(out.getAttributetypeids().get(i));
                						
                						if (out.getEnumeration() != null && out.getEnumeration().trim().length() > 0) {
                        					// Si el valor del atributo pertenece a una enumeración
                							log.debug("getAllServices :: El atributo \"" + out.getAttributetypevalues().get(i) + "\" pertenece a la enumeracion:  \"" + out.getEnumeration() + "\"");
                							JSONObject jsonEnumeration = new JSONObject(out.getEnumeration());
                							Iterator<?> keys = jsonEnumeration.keys();
                							while (keys.hasNext()) {
                								String key = (String)keys.next();
                								if (jsonEnumeration.getString(key).trim().toLowerCase().indexOf(out.getAttributetypevalues().get(i).trim().toLowerCase()) != -1) {
                									if (out.getValue() != null && out.getValue().trim().equalsIgnoreCase(key.trim())) {
                										listIdservices.add(out.getServiceid());
                										listServicesAttributes.add(new ServiceAttributeVO(out.getServiceid(), out.getAttributetypeids().get(i), out.getValue()));
                										break;
                									}
                								}
                							}
                        				} else {
                        					// Si el valor del atributo NO pertenece a una enumeración
                        					log.debug("getAllServices :: El atributo \"" + out.getAttributetypevalues().get(i) + "\" NO pertenece a una enumeracion");
                        					if (out.getValue() != null && out.getValue().trim().toLowerCase().indexOf(out.getAttributetypevalues().get(i).trim().toLowerCase()) != -1) {
                        						listIdservices.add(out.getServiceid());
                        						listServicesAttributes.add(new ServiceAttributeVO(out.getServiceid(), out.getAttributetypeids().get(i), out.getValue()));
                        					}
                        				}
                					}
                				}
                			}
                		}
                		
                		// Recuperar los servicios resultantes de la búsqueda a partir de los ids obtenidos en el primer filtrado
                		List<ServiceAttributeVO> listStep01 = new ArrayList<ServiceAttributeVO>();
                		List<Long> listStep02 = new ArrayList<Long>();
                		if (!listIdservices.isEmpty() && !listAttributeTypes.isEmpty() && !listServicesAttributes.isEmpty()) {
                			listIdservices = deleteDuplicatedIds(listIdservices);
                			listAttributeTypes = deleteDuplicatedIds(listAttributeTypes);
                			final int numAttributeTypes = listAttributeTypes.size();
                			
                			// Cargar lista intermedia de objetos filtrados
                			for (Long type : listAttributeTypes) {
                				for (Long srv : listIdservices) {
                					for (ServiceAttributeVO vo : listServicesAttributes) {
                						if (vo.getAttributetypeid().longValue() == type.longValue() && vo.getServiceid().longValue() == srv.longValue()) {
                							listStep01.add(vo);
                							log.debug("getAllServices :: ServiceAttributeVO ::  " + vo);
                						}
                					}
                				}
                			}
                			
                			// Para que el servicio pase el filtrado, debe aparecer en la lista intermedia "listStep01" tantas
                			// vecer como número de tipos de atributos se estén buscando
                			if (!listStep01.isEmpty()) {
                				for (Long srv : listIdservices) {
                					int cont = 0;
                					for (ServiceAttributeVO vo : listServicesAttributes) {
                						if (srv.longValue() == vo.getServiceid().longValue()) {
                							cont++;
                						}
                					}
                					if (cont == numAttributeTypes) {
                						listStep02.add(srv);
                					}
                				}
                			}
                		} else if (!listIdservices.isEmpty() && listStep02.isEmpty()) {
                			list = serviceRepository.findByMultipleIds(listIdservices);
                		}
                		
                		listStep02 = deleteDuplicatedIds(listStep02);
                		if (!listStep02.isEmpty()) {
                			list = serviceRepository.findByMultipleIds(listStep02);
                		}
                	}
       			}	
        	}
        	return list;
        } catch (Exception e) {
        	log.error("getAllServices  ::  Exception  ::  " + e.getMessage());
        	return null;
        }
    }

    /**
     * GET  /services/:id : get the "id" service.
     *
     * @param id the id of the service to retrieve
     * @return the ResponseEntity with status 200 (OK) and with body the service, or with status 404 (Not Found)
     */
    @GetMapping("/services/{id}")
    @Timed
    public ResponseEntity<Service> getService(@PathVariable Long id) {
    	log.debug("REST request to get Service : {}", id);
        try {
        	log.debug("getService  ::  getting Service...");
        	Service service = serviceRepository.findOne(id);
        	log.debug("getService  ::  getting OK");
            return ResponseUtil.wrapOrNotFound(Optional.ofNullable(service));
        } catch (Exception e) {
        	log.error("getService  ::  Exception  ::  " + e.getMessage());
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(ENTITY_NAME, "serviceGetError", "No puede obtenerse el Servicio"))
                    .body(null);
        }
    }
    
    /**
     * GET  /services-names-only/:id : get the "id" service.
     *
     * @param id the id of the service to retrieve
     * @return the ResponseEntity with status 200 (OK) and with body the service, or with status 404 (Not Found)
     */
    @GetMapping("/services-names-only/{id}")
    @Timed
    public ResponseEntity<String> getServiceName(@PathVariable Long id) {
    	log.debug("REST request to get Service name : {}", id);
        try {
        	log.debug("getService  ::  getting Service name...");
        	String serviceName = serviceRepository.findOne(id).getServicename();
        	log.debug("getService  ::  getting OK");
            return ResponseUtil.wrapOrNotFound(Optional.ofNullable(serviceName));
        } catch (Exception e) {
        	log.error("getService  ::  Exception  ::  " + e.getMessage());
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(ENTITY_NAME, "serviceGetError", "No puede obtenerse el nombre del Servicio"))
                    .body(null);
        }
    }

    /**
     * DELETE  /services/:id : delete the "id" service.
     *
     * @param id the id of the service to delete
     * @return the ResponseEntity with status 200 (OK)
     */
    @DeleteMapping("/services/{id}")
    @Timed
    @Transactional
    public ResponseEntity<Void> deleteService(@PathVariable Long id, @RequestParam String forcedelete) {
    	log.debug("REST request to delete Service : {}", id);
        try {
        	log.debug("deleteService  ::  forcedelete=" + forcedelete);
        	
        	// Obtener el servicio
        	Service service = serviceRepository.findOne(id);
        	boolean markAsErased = false;
        	
        	if (forcedelete != null && !Boolean.valueOf(forcedelete).booleanValue()) {
        		// Marcar el servicio como borrado
            	if (service != null) {
    	        	log.debug("deleteService  ::  soft deleting Service...");
    	        	
    	        	// Marcar atributos del servicio como borrados
    	        	serviceAttributeValueRepository.updateDeletedDateServiceAttributeValueByIdService(ZonedDateTime.now(), id);
    	        	
    	        	// Marcar Servicio como borrado
    	        	markAsErased = true;
    	        	service.setDeleteddate(ZonedDateTime.now());
    	        	serviceRepository.saveAndFlush(service);
            	}
        	} else {
        		// Eliminar definitivamente los atributos del servicio
        		serviceAttributeValueRepository.deleteServiceAttributeValueByIdService(id);
        		
        		// Eliminar definitivamente el servicio
        		log.debug("deleteService  ::  deleting service...");
            	serviceRepository.delete(id);
        	}
        	log.debug("deleteService  ::  deleting OK");
        	
            return markAsErased
            		? ResponseEntity.ok().headers(HeaderUtil.createEntitySoftDeletionAlert(ENTITY_NAME, service.getServicename())).build()
            				: ResponseEntity.ok().headers(HeaderUtil.createEntityDeletionAlert(ENTITY_NAME, service.getServicename())).build();
        } catch (DataIntegrityViolationException e) {
        	log.error("deleteService DataIntegrityViolationException");
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(ENTITY_NAME, "serviceDeleteDataIntegrityError", "El Servicio no puede ser eliminado porque esta siendo utilizado en Contratos"))
                    .body(null);
        } catch (EmptyResultDataAccessException e) {
            return ResponseEntity.noContent().build();
        } catch (Exception e) {
        	log.error("deleteService  ::  Exception  ::  " + e.getMessage());
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(ENTITY_NAME, "serviceDeleteError", "El Servicio no puede ser eliminado"))
                    .body(null);
        }
    }
    
    
    /**
     * Build a ServiceFilterVO
     * @param filter the filter
     * @return SErviceFilterVO
     */
    public ServiceFilterVO buildFilter(final String filter) {
    	ServiceFilterVO vo = null;
    	try {
    		log.debug("buildFilter  ::  " + filter);
    		String[] params = filter.split(PARAM_SEPARATOR);
    		if (params != null && params.length > 0) {
    			vo = new ServiceFilterVO(new Long(params[0].trim()));
    			if (params.length > 1) {
        			for (int i = 1;i < params.length;i++) {
        				String[] split = params[i].trim().split(ATTRIBUTE_SEPARATOR);
        				vo.getAttributetypeids().add(new Long(split[0].trim().substring(1)));
        				vo.getAttributetypevalues().add(split[1].trim().substring(1, split[1].length() - 1));
            		}
        		}
    		}
    	} catch (Exception e) {
    		log.error("buildFilter  ::  Exception  ::  " + e.getMessage());
    	}
    	return vo;
    }
    
    
    /**
     * Call service filter query.
     * @param vo the ServiceFilterVO object
     * @return List
     */
    public List<ServiceFilterVO> filterQuery(ServiceFilterVO vo) {
    	List<ServiceFilterVO> result = new ArrayList<ServiceFilterVO>();
    	try {
        	log.debug("filterQuery  ::  executing...");
        	List<ServiceFilterVO> firstResult = null;
        	if (vo.getAttributetypeids() == null || vo.getAttributetypeids().isEmpty()) {
        		firstResult = serviceRepository.filterQuery(vo.getClassid());
        		log.debug("  *** Resultado : (" + (firstResult == null || firstResult.isEmpty() ? "0" : firstResult.size()) + ") ***");
        		result = firstResult;
        	} else {
        		firstResult = serviceRepository.filterQuery(vo.getClassid(), vo.getAttributetypeids());
        		log.debug("  *** Resultado : (" + (firstResult == null || firstResult.isEmpty() ? "0" : firstResult.size()) + ") ***");
        		result = firstResult;
        	}
        	
    	} catch (Exception e) {
    		log.error("filterQuery  ::  Exception  ::  " + e.getMessage());
    	}
    	return result;
    }
    
    
    /**
     * Delete duplicated ids.
     * @param sourceList
     * @return List
     */
    private List<Long> deleteDuplicatedIds(List<Long> sourceList) {
    	Set<Long> set = new HashSet<Long>();
    	try {
    		log.debug("deleteDuplicatedIds  ::  executing...");
    		set.addAll(sourceList);
        	sourceList.clear();
        	sourceList.addAll(set);
    	} catch (Exception e) {
    		log.error("deleteDuplicatedIds  ::  Exception  ::  " + e.getMessage());
    	}
    	return sourceList;
    }
}
